@@ -16,7 +16,7 @@
 #define MENU_ITEM_WITH_STRING(x,y,z)	item = gtk_menu_item_new_with_label(x); gtk_widget_show (item); gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC(y), (gpointer) z); gtk_menu_shell_append(GTK_MENU_SHELL(sub), item);
 #define MENU_ITEM_WITH_POINTER(x,y,z)	item = gtk_menu_item_new_with_label(x); gtk_widget_show (item); gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC(y), (gpointer) z); gtk_menu_shell_append(GTK_MENU_SHELL(sub), item);
 #define MENU_ITEM(x,y)			item = gtk_menu_item_new_with_label(x); gtk_widget_show (item); gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC(y), NULL); gtk_menu_shell_append(GTK_MENU_SHELL(sub), item);
-#define CHECK_MENU_ITEM(x,y)		item = gtk_check_menu_item_new_with_label(x); gtk_widget_show (item); gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC(y), NULL); gtk_menu_shell_append(GTK_MENU_SHELL(sub), item);
+#define CHECK_MENU_ITEM(x,y,b)		item = gtk_check_menu_item_new_with_label(x); gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), b); gtk_widget_show (item); gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC(y), NULL); gtk_menu_shell_append(GTK_MENU_SHELL(sub), item);
 #define MENU_ITEM_SET_ACCEL(a)		gtk_widget_add_accelerator(item, "activate", accel_group, a, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 #define MENU_TEAROFF			item = gtk_tearoff_menu_item_new(); gtk_widget_show (item); gtk_menu_shell_append(GTK_MENU_SHELL(sub), item);
 #define MENU_SEPARATOR			item = gtk_separator_menu_item_new(); gtk_widget_show (item); gtk_menu_shell_append(GTK_MENU_SHELL(sub), item);
@@ -41,6 +41,21 @@ enum {
 	UNIT_CLASS_COMPOSITE_UNIT
 };
 
+enum {
+	NAMES_NAME_COLUMN,
+	NAMES_ABBREVIATION_STRING_COLUMN,
+	NAMES_PLURAL_STRING_COLUMN,
+	NAMES_REFERENCE_STRING_COLUMN,
+	NAMES_ABBREVIATION_COLUMN,
+	NAMES_SUFFIX_COLUMN,
+	NAMES_UNICODE_COLUMN,
+	NAMES_PLURAL_COLUMN,
+	NAMES_REFERENCE_COLUMN,
+	NAMES_AVOID_INPUT_COLUMN,
+	NAMES_CASE_SENSITIVE_COLUMN,
+	NAMES_N_COLUMNS
+};
+
 enum {
 	DELIMITER_COMMA,
 	DELIMITER_TABULATOR,	
@@ -52,7 +67,8 @@ enum {
 enum {
 	MENU_ARGUMENT_TYPE_FREE,
 	MENU_ARGUMENT_TYPE_NUMBER,	
-	MENU_ARGUMENT_TYPE_INTEGER,		
+	MENU_ARGUMENT_TYPE_INTEGER,
+	MENU_ARGUMENT_TYPE_SYMBOLIC,
 	MENU_ARGUMENT_TYPE_TEXT,
 	MENU_ARGUMENT_TYPE_DATE,	
 	MENU_ARGUMENT_TYPE_VECTOR,	
@@ -70,7 +86,8 @@ enum {
 	MENU_ARGUMENT_TYPE_VARIABLE,
 	MENU_ARGUMENT_TYPE_FILE,
 	MENU_ARGUMENT_TYPE_ANGLE,
-	MENU_ARGUMENT_TYPE_GIAC
+	MENU_ARGUMENT_TYPE_DATA_OBJECT,
+	MENU_ARGUMENT_TYPE_DATA_PROPERTY
 };
 
 enum {
@@ -106,12 +123,20 @@ void create_main_window (void);
 GtkWidget* get_functions_dialog (void);
 GtkWidget* get_variables_dialog (void);
 GtkWidget* get_units_dialog (void);
+GtkWidget* get_datasets_dialog (void);
 GtkWidget* get_preferences_dialog (void);
 GtkWidget* get_unit_edit_dialog (void);
 GtkWidget* get_function_edit_dialog (void);
 GtkWidget* get_variable_edit_dialog (void);
+GtkWidget* get_unknown_edit_dialog (void);
 GtkWidget* get_matrix_edit_dialog (void);
+GtkWidget* get_dataobject_edit_dialog (void);
+GtkWidget* get_dataset_edit_dialog (void);
+GtkWidget* get_dataproperty_edit_dialog (void);
+GtkWidget* get_names_edit_dialog (void);
 GtkWidget* get_csv_import_dialog (void);
+GtkWidget* get_csv_export_dialog (void);
+GtkWidget* get_number_base_expression_dialog (void);
 GtkWidget* get_nbases_dialog (void);
 GtkWidget* get_about_dialog (void);
 GtkWidget* get_argument_rules_dialog (void);
@@ -119,6 +144,7 @@ GtkWidget* get_decimals_dialog (void);
 GtkWidget* get_plot_dialog (void);
 GtkWidget* get_precision_dialog (void);
 GtkWidget* get_unit_dialog (void);
+GtkWidget* get_periodic_dialog (void);
 GtkWidget* create_InfoWidget(const gchar *text);
 
 #endif /* INTERFACE_H */
