@@ -27,19 +27,9 @@ enum {
 enum {
 	FRACTIONAL_MODE_DECIMAL = 1,
 	FRACTIONAL_MODE_COMBINED = 2,	
-	FRACTIONAL_MODE_FRACTION = 3,	
+	FRACTIONAL_MODE_FRACTION = 3
 };
 
-enum {
-	BASE_DECI = 1,
-	BASE_BIN = 2,		
-	BASE_OCTAL = 3,
-	BASE_HEX = 4,
-	BASE_SEXA = 5,
-	BASE_TIMEF = 6,
-	BASE_ROMAN = 7
-};
-
-#include "qalculate.h"
+#include <libqalculate/qalculate.h>
 
 #endif /* MAIN_H */
